@@ -1,8 +1,8 @@
 /*
-    $Id: vcd-info.c,v 1.15 2004/12/01 13:04:59 rocky Exp $
+    $Id: vcd-info.c,v 1.23 2005/05/08 08:42:09 rocky Exp $
 
     Copyright (C) 2001, 2002 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2002, 2003, 2004 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2002, 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@
 #include "bitvec.h"
 #include "pbc.h"
 
-static const char _rcsid[] = "$Id: vcd-info.c,v 1.15 2004/12/01 13:04:59 rocky Exp $";
+static const char _rcsid[] = "$Id: vcd-info.c,v 1.23 2005/05/08 08:42:09 rocky Exp $";
 
 static const char DELIM[] = \
 "----------------------------------------" \
@@ -151,6 +151,8 @@ static struct gl_t
 }
 gl;                             /* global variables */
 
+poptContext optCon;
+
 /* end of vars */
 
 
@@ -193,27 +195,27 @@ typedef enum {
 } vcd2_ext_pbc_status_t;
 
 static vcd2_ext_pbc_status_t
-_has_vcd2_ext_pbc (const vcdinfo_obj_t *obj)
+_has_vcd2_ext_pbc (const vcdinfo_obj_t *p_vcdinfo)
 {
   iso9660_stat_t *statbuf;
-  CdIo *img;
+  CdIo_t *p_cdio;
   vcd2_ext_pbc_status_t ret_status;
   
-  if (!vcdinfo_has_pbc(obj))
+  if (!vcdinfo_has_pbc(p_vcdinfo))
     return PBC_VCD2_NO_PBC;
 
-  if (vcdinfo_get_VCD_type(obj) != VCD_TYPE_VCD2)
+  if (vcdinfo_get_VCD_type(p_vcdinfo) != VCD_TYPE_VCD2)
     return PBC_VCD2_NOPE;
 
-  img = vcdinfo_get_cd_image(obj);
-  statbuf = iso9660_fs_stat (img, "EXT/LOT_X.VCD;1");
+  p_cdio = vcdinfo_get_cd_image(p_vcdinfo);
+  statbuf = iso9660_fs_stat (p_cdio, "EXT/LOT_X.VCD;1");
   if (NULL == statbuf)
     return PBC_VCD2_NO_LOT_X;
   if (statbuf->size != ISO_BLOCKSIZE * LOT_VCD_SIZE) {
     ret_status = PBC_VCD2_BAD_LOT_SIZE;
   } else {
     free(statbuf);
-    statbuf = iso9660_fs_stat (img, "EXT/PSD_X.VCD;1");
+    statbuf = iso9660_fs_stat (p_cdio, "EXT/PSD_X.VCD;1");
     if (NULL != statbuf) {
       ret_status = PBC_VCD2_EXT;
     } else {
@@ -227,23 +229,25 @@ _has_vcd2_ext_pbc (const vcdinfo_obj_t *obj)
 
 
 static void
-dump_lot (const vcdinfo_obj_t *obj, bool ext)
+dump_lot (const vcdinfo_obj_t *p_vcdinfo, bool ext)
 {
-  const LotVcd_t *lot = ext ? vcdinfo_get_lot_x(obj) : vcdinfo_get_lot(obj);
+  const LotVcd_t *lot = ext 
+    ? vcdinfo_get_lot_x(p_vcdinfo) : vcdinfo_get_lot(p_vcdinfo);
   
   unsigned n, tmp;
-  const uint16_t max_lid = vcdinfo_get_num_LIDs(obj);
-  unsigned int mult = vcdinfo_get_offset_mult(obj);
+  const uint16_t max_lid = vcdinfo_get_num_LIDs(p_vcdinfo);
+  unsigned int mult = vcdinfo_get_offset_mult(p_vcdinfo);
 
   if (!gl.show.no.header)
     fprintf (stdout, 
-             (vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
-              || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
+             (vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_SVCD 
+              || vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_HQVCD)
              ? "SVCD/LOT.SVD\n"
              : (ext ? "EXT/LOT_X.VCD\n": "VCD/LOT.VCD\n"));
 
   if (lot->reserved)
-    fprintf (stdout, " RESERVED = 0x%4.4x (should be 0x0000)\n", uint16_from_be (lot->reserved));
+    fprintf (stdout, " RESERVED = 0x%4.4x (should be 0x0000)\n", 
+             uint16_from_be (lot->reserved));
 
   for (n = 0; n < LOT_VCD_OFFSETS; n++)
     {
@@ -253,7 +257,8 @@ dump_lot (const vcdinfo_obj_t *obj, bool ext)
             fprintf (stdout, "warning, LID[1] should have offset = 0!\n");
 
           if (n >= max_lid)
-            fprintf (stdout, "warning, the following entry is greater than the maximum lid field in info\n");
+            fprintf (stdout, 
+                     "warning, the following entry is greater than the maximum lid field in info\n");
           fprintf (stdout, " LID[%d]: offset = %d (0x%4.4x)\n", 
                    n + 1, tmp * mult, tmp);
         }
@@ -263,19 +268,25 @@ dump_lot (const vcdinfo_obj_t *obj, bool ext)
 }
 
 static void
-dump_psd (const vcdinfo_obj_t *obj, bool ext)
+dump_psd (const vcdinfo_obj_t *p_vcdinfo, bool ext)
 {
-  CdioListNode *node;
+  CdioListNode_t *node;
   unsigned n = 0;
-  unsigned int mult = vcdinfo_get_offset_mult(obj);
-  const uint8_t *psd = ext 
-    ? vcdinfo_get_psd_x(obj) : vcdinfo_get_psd(obj);
-  CdioList *offset_list = ext 
-    ? vcdinfo_get_offset_x_list(obj) : vcdinfo_get_offset_list(obj);
+  unsigned int mult;
+  uint8_t *psd;
+  CdioList_t *offset_list;
+
+  if (!p_vcdinfo) return;
+  
+  mult = vcdinfo_get_offset_mult(p_vcdinfo);
+  psd  = ext ? vcdinfo_get_psd_x(p_vcdinfo) : vcdinfo_get_psd(p_vcdinfo);
+  offset_list = ext 
+    ? vcdinfo_get_offset_x_list(p_vcdinfo) 
+    : vcdinfo_get_offset_list(p_vcdinfo);
 
   fprintf (stdout, 
-           (vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
-            || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
+           (vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_SVCD 
+            || vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_HQVCD)
            ? "SVCD/PSD.SVD\n"
            : (ext ? "EXT/PSD_X.VCD\n": "VCD/PSD.VCD\n"));
 
@@ -302,14 +313,14 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
                      "  NOI: %d | LID#: %d (rejected: %s)\n"
                      "  prev: %s | next: %s | return: %s\n"
                      "  playtime: %d/15s | wait: %ds | autowait: %ds\n",
-                     n, vcdinfo_ofs2str (obj, ofs->offset, ext),
+                     n, vcdinfo_ofs2str (p_vcdinfo, ofs->offset, ext),
                      pld->noi, lid, 
                      _vcd_bool_str(vcdinfo_is_rejected(uint16_from_be(pld->lid))),
-                     vcdinfo_ofs2str(obj, 
+                     vcdinfo_ofs2str(p_vcdinfo, 
                                      vcdinf_pld_get_prev_offset(pld), ext),
-                     vcdinfo_ofs2str(obj, 
+                     vcdinfo_ofs2str(p_vcdinfo, 
                                      vcdinf_pld_get_next_offset(pld), ext),
-                     vcdinfo_ofs2str(obj, 
+                     vcdinfo_ofs2str(p_vcdinfo, 
                                      vcdinf_pld_get_return_offset(pld),
                                      ext),
                      vcdinf_get_play_time(pld), vcdinf_get_wait_time (pld),
@@ -325,12 +336,17 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
 
         case PSD_TYPE_END_LIST:
           {
-            const PsdEndListDescriptor *d = (const void *) (psd + _rofs);
-            fprintf (stdout, " PSD[%.2d] (%s): end list descriptor\n", n, vcdinfo_ofs2str (obj, ofs->offset, ext));
-            if (vcdinfo_get_VCD_type(obj) != VCD_TYPE_VCD2)
+            const PsdEndListDescriptor_t *d = (const void *) (psd + _rofs);
+            fprintf (stdout, " PSD[%.2d] (%s): end list descriptor\n", n, 
+                     vcdinfo_ofs2str (p_vcdinfo, ofs->offset, ext));
+            if (vcdinfo_get_VCD_type(p_vcdinfo) != VCD_TYPE_VCD2)
               {
-                fprintf (stdout, "  next disc number: %d (if 0 stop PBC handling)\n", d->next_disc);
-                fprintf (stdout, "  change picture item: %s\n", vcdinfo_pin2str (uint16_from_be (d->change_pic)));
+                fprintf (stdout, 
+                         "  next disc number: %d (if 0 stop PBC handling)\n", 
+                         d->next_disc);
+                fprintf (stdout, 
+                         "  change picture item: %s\n", 
+                         vcdinfo_pin2str (uint16_from_be (d->change_pic)));
               }
             fprintf (stdout, "\n");
           }
@@ -351,19 +367,23 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
                      "  default: %s | timeout: %s\n"
                      "  wait: %d secs | loop: %d (delayed: %s)\n"
                      "  play-item: %s\n",
-                     n, vcdinfo_ofs2str (obj, ofs->offset, ext),
+                     n, vcdinfo_ofs2str (p_vcdinfo, ofs->offset, ext),
                      (type == PSD_TYPE_EXT_SELECTION_LIST ? "extended " : ""),
                      *(uint8_t *) &d->flags,
                      vcdinf_get_num_selections(d),
                      vcdinf_get_bsn(d),
                      lid, 
                      _vcd_bool_str (vcdinf_psd_get_lid_rejected(d)),
-                     vcdinfo_ofs2str(obj, vcdinf_psd_get_prev_offset(d), ext),
-                     vcdinfo_ofs2str(obj, vcdinf_psd_get_next_offset(d), ext),
-                     vcdinfo_ofs2str(obj, vcdinf_psd_get_return_offset(d),ext),
-                     vcdinfo_ofs2str(obj, 
+                     vcdinfo_ofs2str(p_vcdinfo, 
+                                     vcdinf_psd_get_prev_offset(d),   ext),
+                     vcdinfo_ofs2str(p_vcdinfo, 
+                                     vcdinf_psd_get_next_offset(d),   ext),
+                     vcdinfo_ofs2str(p_vcdinfo, 
+                                     vcdinf_psd_get_return_offset(d), ext),
+                     vcdinfo_ofs2str(p_vcdinfo, 
                                      vcdinf_psd_get_default_offset(d),ext),
-                     vcdinfo_ofs2str(obj, vcdinf_get_timeout_offset(d), ext),
+                     vcdinfo_ofs2str(p_vcdinfo, vcdinf_get_timeout_offset(d),
+                                     ext),
                      vcdinf_get_timeout_time(d),
                      vcdinf_get_loop_count(d), 
                      _vcd_bool_str (vcdinf_has_jump_delay(d)),
@@ -371,14 +391,14 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
 
             for (i = 0; i < vcdinf_get_num_selections(d); i++)
               fprintf (stdout, "  ofs[%d]: %s\n", i,
-                       vcdinfo_ofs2str (obj, 
+                       vcdinfo_ofs2str (p_vcdinfo, 
                                         vcdinf_psd_get_offset(d, i), 
                                         ext));
 
             if (type == PSD_TYPE_EXT_SELECTION_LIST 
                 || d->flags.SelectionAreaFlag)
               {
-                const PsdSelectionListDescriptorExtended *d2 =
+                const PsdSelectionListDescriptorExtended_t *d2 =
                   (const void *) &(d->ofs[d->nos]);
 
                 fprintf (stdout, "  prev_area: %s | next_area: %s\n",
@@ -400,7 +420,7 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
           break;
         default:
           fprintf (stdout, " PSD[%2d] (%s): unkown descriptor type (0x%2.2x)\n", 
-                   n, vcdinfo_ofs2str (obj, ofs->offset, ext), type);
+                   n, vcdinfo_ofs2str (p_vcdinfo, ofs->offset, ext), type);
 
           fprintf (stdout, "  hexdump: ");
           _hexdump (&psd[_rofs], 24);
@@ -413,16 +433,16 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
 }
 
 static void
-dump_info (vcdinfo_obj_t *obj)
+dump_info (vcdinfo_obj_t *p_vcdinfo)
 {
-  const InfoVcd_t *info = vcdinfo_get_infoVcd(obj);
-  segnum_t num_segments = vcdinfo_get_num_segments(obj);
+  const InfoVcd_t *info = vcdinfo_get_infoVcd(p_vcdinfo);
+  segnum_t num_segments = vcdinfo_get_num_segments(p_vcdinfo);
   int n;
 
   if (!gl.show.no.header)
     fprintf (stdout, 
-             (vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
-              || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
+             (vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_SVCD 
+              || vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_HQVCD)
              ? "SVCD/INFO.SVD\n" 
              : "VCD/INFO.VCD\n");
 
@@ -433,11 +453,13 @@ dump_info (vcdinfo_obj_t *obj)
   if (gl.show.info.prof)
     fprintf (stdout, " system profile tag: 0x%2.2x\n", info->sys_prof_tag);
   if (gl.show.info.album)
-    fprintf (stdout, " album id: `%.16s'\n", vcdinfo_get_album_id(obj));
+    fprintf (stdout, " album id: `%.16s'\n", vcdinfo_get_album_id(p_vcdinfo));
   if (gl.show.info.count)
-    fprintf (stdout, " volume count: %d\n", vcdinfo_get_volume_count(obj));
+    fprintf (stdout, " volume count: %d\n", 
+             vcdinfo_get_volume_count(p_vcdinfo));
   if (gl.show.info.vol)
-    fprintf (stdout, " volume number: %d\n", vcdinfo_get_volume_num(obj));
+    fprintf (stdout, " volume number: %d\n", 
+             vcdinfo_get_volume_num(p_vcdinfo));
 
   if (gl.show.info.pal)
     {
@@ -454,8 +476,8 @@ dump_info (vcdinfo_obj_t *obj)
       
       fprintf (stdout, " flags:\n");
       fprintf (stdout, 
-               ((vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
-                 || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD) 
+               ((vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_SVCD 
+                 || vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_HQVCD) 
                 ? "  reserved1: %s\n"
                 : "  karaoke area: %s\n"),
                _vcd_bool_str (info->flags.reserved1));
@@ -473,12 +495,12 @@ dump_info (vcdinfo_obj_t *obj)
     fprintf (stdout, "  start track #2: %s\n", _vcd_bool_str (info->flags.use_track3));
   if (gl.show.info.pbc) {
     fprintf (stdout, 
-             ((vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
-               || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD) 
+             ((vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_SVCD 
+               || vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_HQVCD) 
               ? "  reserved2: %s\n"
               : "  extended pbc: %s\n"),
              _vcd_bool_str (info->flags.pbc_x));
-    switch (_has_vcd2_ext_pbc(obj))
+    switch (_has_vcd2_ext_pbc(p_vcdinfo))
       {
       case PBC_VCD2_NO_PBC:
         fprintf(stdout, " No PBC info.\n");
@@ -514,7 +536,7 @@ dump_info (vcdinfo_obj_t *obj)
     
   if (gl.show.info.ofm)
     fprintf (stdout, " offset multiplier: 0x%2.2x\n", 
-             vcdinfo_get_offset_mult(obj));
+             vcdinfo_get_offset_mult(p_vcdinfo));
 
   if (gl.show.info.lidn)
     fprintf (stdout, " maximum lid: %d\n",
@@ -526,7 +548,7 @@ dump_info (vcdinfo_obj_t *obj)
   if (gl.show.info.segs)
     for (n = 0; n < num_segments; n++)
       {
-        const lsn_t  lsn = vcdinfo_get_seg_lsn(obj, n);
+        const lsn_t  lsn = vcdinfo_get_seg_lsn(p_vcdinfo, n);
         msf_t  msf;
         char *psz_msf;
 
@@ -535,23 +557,23 @@ dump_info (vcdinfo_obj_t *obj)
         fprintf (stdout, " SEGMENT[%4.4d]: track# 0, LSN %6u "
                  "(MSF %s), %2u sectors\n",
                  n, (unsigned int) lsn, psz_msf, 
-                 (unsigned int) vcdinfo_get_seg_sector_count(obj, n));
+                 (unsigned int) vcdinfo_get_seg_sector_count(p_vcdinfo, n));
         free(psz_msf);
 
         fprintf (stdout, "   audio: %s, video: %s, continuation %s%s %s\n",
-                 vcdinfo_audio_type2str(obj,
-                                        vcdinfo_get_seg_audio_type(obj, n)),
-                 vcdinfo_video_type2str(obj, n),
+                 vcdinfo_audio_type2str(p_vcdinfo,
+                                        vcdinfo_get_seg_audio_type(p_vcdinfo, n)),
+                 vcdinfo_video_type2str(p_vcdinfo, n),
                  _vcd_bool_str (info->spi_contents[n].item_cont),
-                 (vcdinfo_get_VCD_type(obj) == VCD_TYPE_VCD2) 
+                 (vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_VCD2) 
                    ? "" : ",\n   SVCD subtitle (OGT) substream:",
-                 (vcdinfo_get_VCD_type(obj) == VCD_TYPE_VCD2) 
-                   ? "" : vcdinfo_ogt2str(obj, n));
+                 (vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_VCD2) 
+                   ? "" : vcdinfo_ogt2str(p_vcdinfo, n));
       }
   
   if (gl.show.info.start)
-    if (vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD
-        || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
+    if (vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_SVCD
+        || vcdinfo_get_VCD_type(p_vcdinfo) == VCD_TYPE_HQVCD)
       for (n = 0; n < 5; n++)
         fprintf (stdout, " volume start time[%d]: %d secs\n", 
                  n, uint16_from_be (info->playing_time[n]));
@@ -615,9 +637,9 @@ dump_entries (vcdinfo_obj_t *obj)
 static void
 dump_tracks_svd (vcdinfo_obj_t *obj)
 {
-  const TracksSVD *tracks = vcdinfo_get_tracksSVD(obj);
-  const TracksSVD2 *tracks2 = (const void *) &(tracks->playing_time[tracks->tracks]);
-  const TracksSVD_v30 *tracks_v30 = (TracksSVD_v30 *) tracks;
+  const TracksSVD_t     *tracks = vcdinfo_get_tracksSVD(obj);
+  const TracksSVD2_t    *tracks2 = (const void *) &(tracks->playing_time[tracks->tracks]);
+  const TracksSVD_v30_t *tracks_v30 = (TracksSVD_v30_t *) tracks;
 
   unsigned j;
 
@@ -685,7 +707,7 @@ dump_tracks_svd (vcdinfo_obj_t *obj)
    TRACKS.SVCD to exist on the medium.
 */
 static void
-dump_tracks (const CdIo *cdio)
+dump_tracks (const CdIo_t *cdio)
 {
   track_t i;
   track_t num_tracks = cdio_get_num_tracks(cdio);
@@ -711,6 +733,7 @@ dump_tracks (const CdIo *cdio)
       printf("%3d: %s  %06u  leadout\n",
              (int) i, psz_msf,
              (unsigned int) cdio_msf_to_lsn(&msf));
+      free(psz_msf);
       break;
       } else {
       printf("%3d: %s  %06u  %s\n",
@@ -728,7 +751,7 @@ dump_tracks (const CdIo *cdio)
 static void
 dump_scandata_dat (vcdinfo_obj_t *obj)
 {
-  const ScandataDat1 *_sd1 = vcdinfo_get_scandata(obj);
+  const ScandataDat1_t *_sd1 = vcdinfo_get_scandata(obj);
   const uint16_t scandata_count = uint16_from_be (_sd1->scandata_count);
   const uint16_t track_count = uint16_from_be (_sd1->track_count);
   const uint16_t spi_count = uint16_from_be (_sd1->spi_count);
@@ -742,7 +765,7 @@ dump_scandata_dat (vcdinfo_obj_t *obj)
 
   if (_sd1->version == SCANDATA_VERSION_VCD2)
     {
-      const ScandataDat_v2 *_sd_v2 = (ScandataDat_v2 *) _sd1;
+      const ScandataDat_v2_t *_sd_v2 = (ScandataDat_v2_t *) _sd1;
 
       int n;
 
@@ -771,18 +794,18 @@ dump_scandata_dat (vcdinfo_obj_t *obj)
     }
   else if (_sd1->version == SCANDATA_VERSION_SVCD)
     {
-      const ScandataDat2 *_sd2 = 
+      const ScandataDat2_t *_sd2 = 
         (const void *) &_sd1->cum_playtimes[track_count];
 
-      const ScandataDat3 *_sd3 = 
+      const ScandataDat3_t *_sd3 = 
         (const void *) &_sd2->spi_indexes[spi_count];
 
-      const ScandataDat4 *_sd4 = 
+      const ScandataDat4_t *_sd4 = 
         (const void *) &_sd3->mpeg_track_offsets[track_count];
 
       const int scandata_ofs0 = 
-        __cd_offsetof (ScandataDat3, mpeg_track_offsets[track_count])
-        - __cd_offsetof (ScandataDat3, mpeg_track_offsets);
+        __cd_offsetof (ScandataDat3_t, mpeg_track_offsets[track_count])
+        - __cd_offsetof (ScandataDat3_t, mpeg_track_offsets);
 
       int n;
 
@@ -828,8 +851,11 @@ dump_scandata_dat (vcdinfo_obj_t *obj)
 
           if (!gl.debug_level >= 1
               && n > PRINTED_POINTS
-              && n < scandata_count - PRINTED_POINTS)
+              && n < scandata_count - PRINTED_POINTS) {
+            free(psz_msf);
             continue;
+          }
+          
 
           fprintf (stdout, 
                    "  scanpoint[%.4d] (ofs:%5d): LSN %lu (MSF %s)\n",
@@ -850,7 +876,7 @@ dump_scandata_dat (vcdinfo_obj_t *obj)
 static void
 dump_search_dat (vcdinfo_obj_t *obj)
 {
-  const SearchDat *searchdat = vcdinfo_get_searchDat(obj);
+  const SearchDat_t *searchdat = vcdinfo_get_searchDat(obj);
   unsigned m;
   uint32_t scan_points = uint16_from_be (searchdat->scan_points);
 
@@ -896,10 +922,10 @@ dump_search_dat (vcdinfo_obj_t *obj)
 static void
 _dump_fs_recurse (const vcdinfo_obj_t *obj, const char pathname[])
 {
-  CdioList *entlist;
-  CdioList *dirlist =  _cdio_list_new ();
-  CdioListNode *entnode;
-  CdIo *cdio = vcdinfo_get_cd_image(obj);
+  CdioList_t *entlist;
+  CdioList_t *dirlist =  _cdio_list_new ();
+  CdioListNode_t *entnode;
+  CdIo_t *cdio = vcdinfo_get_cd_image(obj);
 
   entlist = iso9660_fs_readdir (cdio, pathname, true);
     
@@ -974,9 +1000,9 @@ dump_fs (vcdinfo_obj_t *obj)
 }
 
 static void
-dump_pvd (vcdinfo_obj_t *obj)
+dump_pvd (vcdinfo_obj_t *p_vcdinfo)
 {
-  const iso9660_pvd_t *pvd = vcdinfo_get_pvd(obj);
+  const iso9660_pvd_t *pvd = vcdinfo_get_pvd(p_vcdinfo);
 
   if (!gl.show.no.header)
     fprintf (stdout, "ISO9660 primary volume descriptor\n");
@@ -995,24 +1021,38 @@ dump_pvd (vcdinfo_obj_t *obj)
   if (gl.show.pvd.vers)
     fprintf (stdout, " version: %d\n", iso9660_get_pvd_version(pvd));
 
-  if (gl.show.pvd.sys)
-    fprintf (stdout, " system id: `%s'\n",    vcdinfo_get_system_id(obj));
+  if (gl.show.pvd.sys) {
+    char *psz = vcdinfo_get_system_id(p_vcdinfo);
+    fprintf (stdout, " system id: `%s'\n",    psz);
+    free(psz);
+  }
+  
 
   if (gl.show.pvd.vol)
-    fprintf (stdout, " volume id: `%s'\n",    vcdinfo_get_volume_id(obj));
+    fprintf (stdout, " volume id: `%s'\n",
+             vcdinfo_get_volume_id(p_vcdinfo));
 
   if (gl.show.pvd.volset)
-    fprintf (stdout, " volumeset id: `%s'\n", vcdinfo_get_volumeset_id(obj));
+    fprintf (stdout, " volumeset id: `%s'\n",
+             vcdinfo_get_volumeset_id(p_vcdinfo));
 
-  if (gl.show.pvd.pub)
-    fprintf (stdout, " publisher id: `%s'\n", vcdinfo_get_publisher_id(obj));
+  if (gl.show.pvd.pub) {
+    char *psz = vcdinfo_get_publisher_id(p_vcdinfo);
+    fprintf (stdout, " publisher id: `%s'\n", psz);
+    free(psz);
+  }
 
-  if (gl.show.pvd.prep)
-    fprintf (stdout, " preparer id: `%s'\n",  vcdinfo_get_preparer_id(obj));
+  if (gl.show.pvd.prep) {
+    char *psz = vcdinfo_get_preparer_id(p_vcdinfo);
+    fprintf (stdout, " preparer id: `%s'\n",  psz);
+    free(psz);
+  }
 
-  if (gl.show.pvd.app)
-    fprintf (stdout, " application id: `%s'\n", 
-             vcdinfo_get_application_id(obj));
+  if (gl.show.pvd.app) {
+    char *psz = vcdinfo_get_application_id(p_vcdinfo);
+    fprintf (stdout, " application id: `%s'\n", psz);
+    free(psz);
+  }
   
   if (gl.show.pvd.iso)
     fprintf (stdout, " ISO size: %d blocks (logical blocksize: %d bytes)\n", 
@@ -1021,93 +1061,97 @@ dump_pvd (vcdinfo_obj_t *obj)
 
   if (gl.show.pvd.xa) 
     fprintf (stdout, " XA marker present: %s\n", 
-             _vcd_bool_str (vcdinfo_has_xa(obj)));
+             _vcd_bool_str (vcdinfo_has_xa(p_vcdinfo)));
 }
 
 static void
-dump_all (vcdinfo_obj_t *obj)
+dump_all (vcdinfo_obj_t *p_vcdinfo)
 {
-  CdIo *cdio = vcdinfo_get_cd_image(obj);
+  CdIo_t *p_cdio;
+
+  if (!p_vcdinfo) return;
+  
+  p_cdio = vcdinfo_get_cd_image(p_vcdinfo);
 
   if (gl.show.pvd.any) 
     {
       if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-      dump_pvd (obj);
+      dump_pvd (p_vcdinfo);
     }
   
   if (gl.show.fs) 
     {
       if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-      dump_fs (obj);
+      dump_fs (p_vcdinfo);
     }
 
   if (gl.show.info.any) 
     {
       if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-      dump_info (obj);
+      dump_info (p_vcdinfo);
     }
   
   if (gl.show.entries.any) 
     {
       if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-      dump_entries (obj);
+      dump_entries (p_vcdinfo);
     }
 
   if (gl.show.psd) 
     {
-      if (vcdinfo_get_psd_size (obj))
+      if (vcdinfo_get_psd_size (p_vcdinfo))
         {
-          vcdinfo_visit_lot (obj, false);
+          vcdinfo_visit_lot (p_vcdinfo, false);
           if (gl.show.lot)
             {
               if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-              dump_lot (obj, false);
+              dump_lot (p_vcdinfo, false);
             }
           
           if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-          dump_psd (obj, false);
+          dump_psd (p_vcdinfo, false);
         }
       
-      if (vcdinfo_get_psd_x_size(obj) && ! gl.no_ext_psd_flag )
+      if (vcdinfo_get_psd_x_size(p_vcdinfo) && ! gl.no_ext_psd_flag )
         {
-          vcdinfo_visit_lot (obj, true);
+          vcdinfo_visit_lot (p_vcdinfo, true);
           if (gl.show.lot) 
             {
               if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-              dump_lot (obj, true);
+              dump_lot (p_vcdinfo, true);
             }
           if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-          dump_psd (obj, true);
+          dump_psd (p_vcdinfo, true);
         }
     }
 
   if (gl.show.tracks) 
     {
-      if (vcdinfo_get_tracksSVD(obj))
+      if (vcdinfo_get_tracksSVD(p_vcdinfo))
         {
           if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-          dump_tracks_svd (obj);
+          dump_tracks_svd (p_vcdinfo);
         } 
       if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-      dump_tracks (cdio);
+      dump_tracks (p_cdio);
     }
   
   if (gl.show.search) 
     {
-      if (vcdinfo_get_searchDat(obj))
+      if (vcdinfo_get_searchDat(p_vcdinfo))
         {
           if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-          dump_search_dat (obj);
+          dump_search_dat (p_vcdinfo);
         }
     }
   
 
   if (gl.show.scandata) 
     {
-      if (vcdinfo_get_scandata(obj))
+      if (vcdinfo_get_scandata(p_vcdinfo))
         {
           if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-          dump_scandata_dat (obj);
+          dump_scandata_dat (p_vcdinfo);
         }
       
     }
@@ -1119,9 +1163,9 @@ static void
 dump (char *image_fname[])
 {
   unsigned size, psd_size;
-  vcdinfo_obj_t *obj;
-  CdIo *img;
-  iso9660_stat_t *statbuf;
+  vcdinfo_obj_t *obj = NULL;
+  CdIo_t *p_cdio;
+  iso9660_stat_t *p_statbuf;
   vcdinfo_open_return_t open_rc;
   
   if (!gl.show.no.banner)
@@ -1141,20 +1185,20 @@ dump (char *image_fname[])
     } else {
       fprintf (stdout, "Error opening requested Video CD object %s\n",
                *image_fname);
-      fprintf (stdout, "Perhaps this is not a Video CD.n");
+      fprintf (stdout, "Perhaps this is not a Video CD\n");
       free(*image_fname);
     }
-    exit (EXIT_FAILURE);
+    goto err_exit;
   }
 
-  img = vcdinfo_get_cd_image(obj);
-  if (open_rc==VCDINFO_OPEN_ERROR || img == NULL) {
+  p_cdio = vcdinfo_get_cd_image(obj);
+  if (open_rc==VCDINFO_OPEN_ERROR || p_cdio == NULL) {
     vcd_error ("Error determining place to read from");
     free(*image_fname);
-    exit (EXIT_FAILURE);
+    goto err_exit;
   }
 
-  size = cdio_stat_size (img);
+  size = cdio_stat_size (p_cdio);
 
   if (gl.show.source) 
     {
@@ -1184,14 +1228,14 @@ dump (char *image_fname[])
 
     if (gl.show.tracks) {
       if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-      dump_tracks (img);
+      dump_tracks (p_cdio);
     }
-    exit (EXIT_FAILURE);
+    goto err_exit;
   }
 
   if (vcdinfo_get_format_version (obj) == VCD_TYPE_INVALID) {
     vcd_error ("VCD detection failed - aborting");
-    exit (EXIT_FAILURE);
+    goto err_exit;
   } else if (gl.show.format) {
     fprintf (stdout, "%s detected\n", vcdinfo_get_format_version_str(obj));
   }
@@ -1201,28 +1245,33 @@ dump (char *image_fname[])
   if (vcdinfo_read_psd (obj))
     {
       /* ISO9660 crosscheck */
-      statbuf = iso9660_fs_stat (img, 
+      p_statbuf = iso9660_fs_stat (p_cdio, 
                                  ((vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
                              || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
                                   ? "/SVCD/PSD.SVD;1" 
                                   : "/VCD/PSD.VCD;1"));
-      if (NULL == statbuf)
+      if (!p_statbuf)
         vcd_warn ("no PSD file entry found in ISO9660 fs");
       else {
-        if (psd_size != statbuf->size)
+        if (psd_size != p_statbuf->size)
           vcd_warn ("ISO9660 psd size != INFO psd size");
-        if (statbuf->lsn != PSD_VCD_SECTOR)
+        if (p_statbuf->lsn != PSD_VCD_SECTOR)
           vcd_warn ("psd file entry in ISO9660 not at fixed LSN");
-        free(statbuf);
+        free(p_statbuf);
       }
           
     }
 
   dump_all (obj);
   vcdinfo_close(obj);
+  return;
+  
+ err_exit:
+  poptFreeContext(optCon);
+  exit (EXIT_FAILURE);
 }
 
-static vcd_log_handler_t  gl_default_vcd_log_handler = NULL;
+static vcd_log_handler_t  gl_default_vcd_log_handler  = NULL;
 static cdio_log_handler_t gl_default_cdio_log_handler = NULL;
 
 static void 
@@ -1335,6 +1384,7 @@ main (int argc, const char *argv[])
   int terse_flag       = false;
   int sector_2336_flag = 0;
   char *source_name    = NULL;
+  const char **args    = NULL;
 
   int opt;
   char *opt_arg;
@@ -1488,7 +1538,7 @@ main (int argc, const char *argv[])
     {"xa",    &gl.show.pvd.xa},
   };
 
-  poptContext optCon = poptGetContext (NULL, argc, argv, optionsTable, 0);
+  optCon = poptGetContext (NULL, argc, argv, optionsTable, 0);
 
   init();
 
@@ -1585,11 +1635,24 @@ main (int argc, const char *argv[])
         exit (EXIT_FAILURE);
       }
 
-  if (poptGetArgs (optCon) != NULL)
+  if ((args = poptGetArgs (optCon)) != NULL)
     {
-      fprintf (stderr, "error - no arguments expected! - try --help\n");
-      poptFreeContext(optCon);
-      exit (EXIT_FAILURE);
+      if (args[1]) {
+        fprintf ( stderr, "too many arguments - try --help");
+        poptFreeContext(optCon);
+        exit (EXIT_FAILURE);
+      }
+
+      if (source_name) {
+        fprintf ( stderr, 
+                  "source file specified as an option and without "
+                  " - try --help\n");
+        poptFreeContext(optCon);
+        exit (EXIT_FAILURE);
+      }
+      
+      source_name    = strdup(args[0]);
+      gl.source_type = OP_SOURCE_UNDEF;
     }
 
   if (gl.debug_level == 3) {
