@@ -1,5 +1,5 @@
 /*
-    $Id: vcd_xml_build.c,v 1.26 2005/05/07 19:53:21 rocky Exp $
+    $Id: vcd_xml_build.c,v 1.27 2005/06/09 00:53:23 rocky Exp $
 
     Copyright (C) 2001, 2003, 2005 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -49,7 +49,7 @@
 #include "vcd_xml_dtd.h"
 #include "vcd_xml_common.h"
 
-static const char _rcsid[] = "$Id: vcd_xml_build.c,v 1.26 2005/05/07 19:53:21 rocky Exp $";
+static const char _rcsid[] = "$Id: vcd_xml_build.c,v 1.27 2005/06/09 00:53:23 rocky Exp $";
 
 static void
 _init_xml (void)
@@ -353,10 +353,10 @@ _do_cl (int argc, const char *argv[])
   return 0;
 }
 
-static VcdImageSink *
+static VcdImageSink_t *
 _create_sink (void)
 {
-  VcdImageSink *image_sink = NULL;
+  VcdImageSink_t *image_sink = NULL;
   CdioListNode_t *node;
 
   switch (gl.img_type)
@@ -446,7 +446,7 @@ main (int argc, const char *argv[])
     xmlNodePtr root;
     xmlNsPtr ns;
     vcdxml_t vcdxml;
-    VcdImageSink *image_sink;
+    VcdImageSink_t *image_sink;
 
     vcd_xml_init (&vcdxml);
     
