@@ -1,5 +1,5 @@
 /*
-    $Id: vcd_xml_parse.h,v 1.3 2003/11/10 11:57:48 rocky Exp $
+    $Id: vcd_xml_parse.h,v 1.4 2005/05/07 19:53:21 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -24,7 +24,7 @@
 #include "vcdxml.h"
 #include <libxml/tree.h>
 
-bool vcd_xml_parse (struct vcdxml_t *obj, xmlDocPtr doc, xmlNodePtr node, xmlNsPtr ns);
+bool vcd_xml_parse (vcdxml_t *obj, xmlDocPtr doc, xmlNodePtr node, xmlNsPtr ns);
 
 #endif /* __VCD_XML_PARSE_H__ */
 
