@@ -1,5 +1,5 @@
 /*
-    $Id: pbc.c,v 1.10 2005/05/08 03:48:55 rocky Exp $
+    $Id: pbc.c,v 1.11 2005/06/18 03:27:24 rocky Exp $
 
     Copyright (C) 2000, 2004, 2005 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -45,7 +45,7 @@
 #include "pbc.h"
 #include "util.h"
 
-static const char _rcsid[] = "$Id: pbc.c,v 1.10 2005/05/08 03:48:55 rocky Exp $";
+static const char _rcsid[] = "$Id: pbc.c,v 1.11 2005/06/18 03:27:24 rocky Exp $";
 
 static uint8_t
 _wtime (int seconds)
@@ -802,7 +802,7 @@ _vcd_pbc_node_write (const VcdObj_t *obj, const pbc_t *p_pbc, void *buf,
 }
 
 pbc_t *
-vcd_pbc_new (enum pbc_type_t type)
+vcd_pbc_new (pbc_type_t type)
 {
   pbc_t *p_pbc;
 
