@@ -1,5 +1,5 @@
 /*
-    $Id: stream_stdio.h,v 1.2 2003/11/10 11:57:50 rocky Exp $
+    $Id: stream_stdio.h,v 1.3 2005/06/07 23:29:23 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -28,7 +28,7 @@
 VcdDataSink*
 vcd_data_sink_new_stdio(const char pathname[]);
 
-VcdDataSource*
+VcdDataSource_t *
 vcd_data_source_new_stdio(const char pathname[]);
 
 #endif /* __VCD_STREAM_STDIO_H__ */
